#define G_LOG_DOMAIN "ap-skill"

#include "ap-skill.h"

#include <glib.h>

typedef struct _APSkillPrivate
{
    gchar    *name;
    gchar    *label;
    guint     score;
    gchar    *modifier;
    guint     bonus;
    gboolean  proficient;
} APSkillPrivate;

struct _APSkill
{
    GObject parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (APSkill, ap_skill, G_TYPE_OBJECT)

enum
{
    PROP_0,
    PROP_NAME,
    PROP_LABEL,
    PROP_SCORE,
    PROP_MODIFIER,
    PROP_BONUS,
    PROP_PROFICIENT,
    N_PROPERTIES
};

static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };


static void
ap_skill_set_property (GObject      *object,
                           guint         property_id,
                           const GValue *value,
                           GParamSpec   *pspec)
{
    APSkill *self = AP_SKILL (object);
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    switch (property_id)
    {
        case PROP_NAME:
            priv->name = g_value_dup_string (value);
            break;

        case PROP_LABEL:
            priv->label = g_value_dup_string (value);
            break;

        case PROP_SCORE:
            priv->score = g_value_get_int (value);
            break;

        case PROP_MODIFIER:
            priv->modifier = g_value_dup_string (value);
            break;

        case PROP_BONUS:
            priv->bonus = g_value_get_uint (value);
            break;

        case PROP_PROFICIENT:
            priv->proficient = g_value_get_boolean (value);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
ap_skill_get_property (GObject      *object,
                           guint         property_id,
                           GValue       *value,
                           GParamSpec   *pspec)
{
    APSkill *self = AP_SKILL (object);
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    switch (property_id)
    {
        case PROP_NAME:
            g_value_set_string (value, priv->name);
            break;

        case PROP_LABEL:
            g_value_set_string (value, priv->label);
            break;

        case PROP_SCORE:
            g_value_set_int (value, priv->score);
            break;

        case PROP_MODIFIER:
            g_value_set_string (value, priv->modifier);
            break;

        case PROP_BONUS:
            g_value_set_uint (value, priv->bonus);
            break;

        case PROP_PROFICIENT:
            g_value_set_boolean (value, priv->proficient);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
ap_skill_finalize (GObject *object)
{
   // APSkillPrivate *priv = ap_skill_get_instance_private (AP_SKILL (object));

    G_OBJECT_CLASS (ap_skill_parent_class)->finalize (object);
}

static void
ap_skill_dispose (GObject *object)
{
    //APSkillPrivate *priv = ap_skill_get_instance_private (AP_SKILL (object));

    G_OBJECT_CLASS (ap_skill_parent_class)->finalize (object);
}

static void
ap_skill_init (APSkill *self)
{
    //APSkillPrivate *priv = ap_skill_get_instance_private (self);
}

static void
ap_skill_class_init (APSkillClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->set_property = ap_skill_set_property;
    object_class->get_property = ap_skill_get_property;
    object_class->finalize = ap_skill_finalize;
    object_class->dispose = ap_skill_dispose;

    obj_properties[PROP_NAME] =
        g_param_spec_string ("name",
                             "name",
                             "Name of the skill.",
                             NULL  /* default value */,
                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);

    obj_properties[PROP_LABEL] =
        g_param_spec_string ("label",
                             "label",
                             "Display name of the skill.",
                             NULL, /* default value */
                             G_PARAM_READWRITE);

    obj_properties[PROP_SCORE] =
        g_param_spec_uint ("score",
                           "score",
                           "The skill score level.",
                           0,
                           100,
                           0, /* default value */
                           G_PARAM_READWRITE);

    obj_properties[PROP_MODIFIER] =
        g_param_spec_string ("modifier",
                             "modifier",
                             "The skill that modifies the skill.",
                             NULL, /* default value */
                             G_PARAM_READWRITE);

    obj_properties[PROP_BONUS] =
        g_param_spec_uint ("bonus",
                           "bonus",
                           "The skill bonus.",
                           0,
                           100,
                           0, /* default value */
                           G_PARAM_READWRITE);

    obj_properties[PROP_PROFICIENT] =
        g_param_spec_boolean ("proficient",
                              "proficient",
                              "Is the character proficient with this skill?",
                              FALSE, /* default value */
                              G_PARAM_READWRITE);

    g_object_class_install_properties (object_class,
                                       N_PROPERTIES,
                                       obj_properties);
}

/*
 * *****************************************************************************
 *
 * API
 *
 * *****************************************************************************
 */

APSkill *
ap_skill_new (void)
{
    return g_object_new (AP_TYPE_SKILL,
                         NULL);
}

gchar *
ap_skill_get_name (APSkill *self)
{
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    return priv->name;
}

gchar *
ap_skill_get_label (APSkill *self)
{
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    return priv->label;
}

guint
ap_skill_get_score (APSkill *self)
{
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    return priv->score;
}

gchar *
ap_skill_get_modifier (APSkill *self)
{
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    return priv->modifier;
}

guint
ap_skill_get_bonus (APSkill *self)
{
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    return priv->bonus;
}

gboolean
ap_skill_get_proficient (APSkill *self)
{
    APSkillPrivate *priv = ap_skill_get_instance_private (self);

    return priv->proficient;
}
