#define G_LOG_DOMAIN "ap-character"

#include "character.h"
#include "sheet.h"
#include "game.h"
#include "bio.h"
#include "stats.h"
#include "attacks.h"
#include "inventory.h"

#include <glib.h>
#include <glib-object.h>

#include <libxml/xmlreader.h>
#include <libxml/xmlwriter.h>

//TODO: Needed?
#include <libxml/tree.h>
#include <libxml/parser.h>

typedef struct _APCharacterPrivate
{
    APSheetXml *sheet;
} APCharacterPrivate;

struct _APCharacter
{
    GObject parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (APCharacter, ap_character, G_TYPE_OBJECT)

/*
 * boilerplate_window_constructed:
 * @object: An instance of #APCharacter
 *
 * The constructed virtual function sets first-run settings.
 */
static void
ap_character_constructed (GObject *object)
{
    APCharacter *self = AP_CHARACTER (object);
    APCharacterPrivate *priv = ap_character_get_instance_private (self);

    G_OBJECT_CLASS (ap_character_parent_class)->constructed (object);
}

/*
 * ap_character_finalize:
 * @object: An instance of #APCharacter
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
ap_character_finalize (GObject *object)
{
    APCharacterPrivate *priv = ap_character_get_instance_private (AP_CHARACTER (object));

    G_OBJECT_CLASS (ap_character_parent_class)->finalize (object);
}

/*
 * ap_character_init:
 * @self: An instance of #APCharacter
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
ap_character_init (APCharacter *self)
{
    APCharacterPrivate *priv = ap_character_get_instance_private (self);
}

/*
 * ap_character_class_init:
 * @klass: An instance of #APCharacterClass
 *
 * Initializes the various virtural functions in #APCharacterClass.
 */
static void
ap_character_class_init (APCharacterClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->constructed = ap_character_constructed;
    object_class->finalize = ap_character_finalize;
}

/**
 * ap_character_new:
 *
 * Creates a new #APCharacter, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * APCharacter *character;
 *
 * character = ap_character_new ();
 * ]|
 *
 * Returns: #APCharacter; a sub-object of #GObject
 */
APCharacter *
ap_character_new (void)
{
    return g_object_new (AP_TYPE_CHARACTER,
                         NULL);
}

/**
 * ap_character_new:
 *
 * Creates a new #APCharacter, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * APCharacter *character;
 *
 * character = ap_character_from_file ("./my-character.xml");
 * ]|
 *
 * Returns: #APCharacter; a sub-object of #GObject
 */
APCharacter *
ap_character_from_file (const gchar *file)
{
    return g_object_new (AP_TYPE_CHARACTER,
						 "file", file,
                         NULL);
}


