#include "test-ability.h"

#include <adventure-pack/ap-ability.h>

#include <glib.h>

void
ability_fixture_set_up (AbilityFixture *fixture,
                        gconstpointer   user_data)
{
    fixture->ability = g_object_new (AP_TYPE_ABILITY,
                                     "name", "example-ability",
                                     NULL);
}

void
ability_fixture_tear_down (AbilityFixture *fixture,
                           gconstpointer   user_data)
{
    g_clear_object (&fixture->ability);
}

void
test_ability_name (AbilityFixture *fixture,
                   gconstpointer   user_data)
{
    gchar *name;

    name = ap_ability_get_name (fixture->ability);

    g_assert_cmpstr (name, ==, "example-ability");
}

void
test_ability_name_changed (AbilityFixture *fixture,
                           gconstpointer   user_data)
{
    gchar *name;

    g_test_expect_message ("ap-ability",
                           G_LOG_LEVEL_WARNING,
                           "Changing the ability name after construct is discouraged. APAbility will not update the name in other widgets.");

    ap_ability_set_name (fixture->ability, "example-ability2");

    g_test_assert_expected_messages ();

    name = ap_ability_get_name (fixture->ability);

    g_assert_cmpstr (name, ==, "example-ability2");
}

void
test_ability_label (AbilityFixture *fixture,
                    gconstpointer   user_data)
{
    gchar *label;

    ap_ability_set_label (fixture->ability, "Example Ability");

    label = ap_ability_get_label (fixture->ability);

    g_assert_cmpstr (label, ==, "Example Ability");
}

void
test_ability_score (AbilityFixture *fixture,
                    gconstpointer   user_data)
{
    GValue val = G_VALUE_INIT;
    gint score;

    g_value_init (&val, G_TYPE_INT);
    g_value_set_int (&val, 10);

    g_object_set_property (G_OBJECT (fixture->ability), "score", &val);

    g_value_unset (&val);

    ap_ability_set_score (fixture->ability, 10);

    score = ap_ability_get_score (fixture->ability);

    g_assert_cmpint (score, ==, 10);
}

void
test_ability_score_with_bonus (AbilityFixture *fixture,
                               gconstpointer   user_data)
{
    GValue val = G_VALUE_INIT;
    GValue val2 = G_VALUE_INIT;
    gint score;

    g_value_init (&val, G_TYPE_INT);
    g_value_init (&val2, G_TYPE_INT);
    g_value_set_int (&val, 10);
    g_value_set_int (&val2, 3);

    g_object_set_property (G_OBJECT (fixture->ability), "score", &val);
    g_object_set_property (G_OBJECT (fixture->ability), "bonus", &val2);

    g_value_unset (&val);
    g_value_unset (&val2);

    score = ap_ability_get_score (fixture->ability);

    g_assert_cmpint (score, ==, 13);
}

void
test_ability_modifier (AbilityFixture *fixture,
                       gconstpointer   user_data)
{
    return;
    GValue val = G_VALUE_INIT;
    gint score;
    gint modifier;

    g_value_init (&val, G_TYPE_INT);
    g_value_set_int (&val, GPOINTER_TO_INT (user_data));

    g_object_set_property (G_OBJECT (fixture->ability), "score", &val);

    g_value_unset (&val);

    score = ap_ability_get_score (fixture->ability);
    modifier = ap_ability_get_modifier (fixture->ability);

    if (score == 1)
        g_assert_cmpint (modifier, ==, -5);
    else if (score == 2 || score == 3)
        g_assert_cmpint (modifier, ==, -4);
    else if (score == 4 || score == 5)
        g_assert_cmpint (modifier, ==, -3);
    else if (score == 6 || score == 7)
        g_assert_cmpint (modifier, ==, -2);
    else if (score == 8 || score == 9)
        g_assert_cmpint (modifier, ==, -1);
    else if (score == 10 || score == 11)
        g_assert_cmpint (modifier, ==, 0);
    else if (score == 12 || score == 13)
        g_assert_cmpint (modifier, ==, 1);
    else if (score == 14 || score == 15)
        g_assert_cmpint (modifier, ==, 2);
    else if (score == 16 || score == 17)
        g_assert_cmpint (modifier, ==, 3);
    else if (score == 18 || score == 19)
        g_assert_cmpint (modifier, ==, 4);
    else if (score == 20 || score == 21)
        g_assert_cmpint (modifier, ==, 5);
    else if (score == 22 || score == 23)
        g_assert_cmpint (modifier, ==, 6);
    else if (score == 24 || score == 25)
        g_assert_cmpint (modifier, ==, 7);
    else if (score == 26 || score == 27)
        g_assert_cmpint (modifier, ==, 8);
    else if (score == 28 || score == 29)
        g_assert_cmpint (modifier, ==, 9);
    else if (score == 30)
        g_assert_cmpint (modifier, ==, 10);
    else
        g_warning ("Score out of 1 - 30 range, is: \"%i\"", score);
}
